unit WavEngU;

interface

uses
  Windows,
  Math,
  WaveObj;

function GetEngineVersion(var VerNr: Word): PChar;
export; stdcall;

function ResetFilePointer(hFile: THandle): LongWord;
export; stdcall;
// Reset file pointer to the begin of file

function MoveFilePointer(hFile: THandle; Dist: LongInt): LongWord;
export; stdcall;
// Moves the file pointer from the current position

function GetFilePointer(hFile: THandle): LongWord;
export; stdcall;
// Returns the current file pointer position

function OpenSampleFile(FileName: PChar): THandle;
export; stdcall;
// opens an existing sample file

function CreateSampleFile(FileName: PChar; AlwaysOverwrite: Boolean): THandle;
export; stdcall;
// creates a new sample file

function ReadWaveHeader(FH: THandle; var WH: TWaveHeader; var Samples: LongWord): LongWord;
export; stdcall;
// reads RIFF header, sample count and required WAVE chunks from WAVE file
// into a TWaveHeader structure and an unsigned long integer;
// returns data offset in WAVE file or 0

function WriteWaveHeader(FH: THandle; var WH: TWaveHeader; Samples: LongWord): LongWord;
export; stdcall;
// writes RIFF header and required WAVE chunks from WAVE file
// from a TWaveHeader structure; calculates expected sizes;
// returns data offset for WAVE file or 0

function ReadIntBlock(FH: THandle; IBuf: PInt; ICount: LongWord; var WH: TWaveHeader): LongWord;
export; stdcall;
// reads a block of up to (ICount) integer sample values from WAVE file
// into a buffer for long integer values;
// returns the real count of read values

function ReadFloatBlock(FH: THandle; FBuf: PFloat; FCount: LongWord): LongWord;
export; stdcall;
// reads a block of up to (FCount) float sample values from WAVE file;
// returns the real count of read values

procedure Int2FloatBlock(IBuf: PInt; FBuf: PFloat; Count: LongWord; ValidBits: Byte);
export; stdcall;

procedure Float2IntBlock(FBuf: PFloat; IBuf: PInt; Count: LongWord; ValidBits: Byte);
export; stdcall;

function WriteIntBlock(FH: THandle; IBuf: PInt; ICount: LongWord; var WH: TWaveHeader): LongWord;
export; stdcall;

function WriteFloatBlock(FH: THandle; FBuf: PFloat; FCount: LongWord): LongWord;
export; stdcall;

function Sign(Value : Single): ShortInt;
export; stdcall;

function dB(r: Single): Single;
export; stdcall;

function dB1(r: Single): Single;
export; stdcall;

function TanH(r: Single): Single;
export; stdcall;

function ArTanH(r: Single): Single;
export; stdcall;

procedure GetValueRange(FBuf: PFloat; Count: LongWord; var Min, Max: Single);
export; stdcall;

procedure ProcessSampleBlock(FBuf: PFloat; Count: LongWord; var Callback: TPerSampleFunction);
export; stdcall;

implementation

{ ... some functions cut out }

function dB(r: Single): Single;
export; stdcall;
begin
  if r < 2e-10 // < 1/(2^32)
  then Result := -256 // to avoid -INF; dB(1/(2^32)) ~ -192
  else Result := 20*Log10(Abs(r))
end;

function dB1(r: Single): Single;
export; stdcall;
begin
  Result := -Power(10, r/20)
end;

function TanH(r: Single): Single;
export; stdcall;
begin
  Result := 1.0-2.0/(1.0+Exp(2.0*r))
end;

function ArTanH(r: Single): Single;
export; stdcall;
begin
  Result := 0.5*ln((1+r)/(1-r))
end;

exports
  GetEngineVersion,
  ResetFilePointer,
  MoveFilePointer,
  GetFilePointer,
  OpenSampleFile,
  CreateSampleFile,
  ReadWaveHeader,
  WriteWaveHeader,
  ReadIntBlock,
  ReadFloatBlock,
  Int2FloatBlock,
  Float2IntBlock,
  WriteIntBlock,
  WriteFloatBlock,
  Sign,
  dB, dB1,
  TanH, ArTanH,
  GetValueRange,
  ProcessSampleBlock;

begin
end.
